@@ -1,38 +1,31 @@
 import click
 import neuro_sdk
 import signal
-import sys
+from contextlib import AsyncExitStack
 from datetime import datetime, timezone
 from dateutil.parser import isoparse
 from neuro_cli.parse_utils import parse_timedelta
 from typing import List, Optional, Sequence, Tuple
 
-from neuro_flow.batch_executor import ExecutorData
 from neuro_flow.batch_runner import BatchRunner
-from neuro_flow.cli.click_types import (
-    BAKE,
-    BATCH,
-    BATCH_OR_ALL,
-    FINISHED_TASK_AFTER_BAKE,
-)
+from neuro_flow.cli.click_types import BAKE, BATCH, BATCH_OR_ALL, BakeTaskType
 from neuro_flow.cli.utils import argument, option, resolve_bake, wrap_async
-from neuro_flow.storage import APIStorage, NeuroStorageFS, Storage
+from neuro_flow.storage.api import ApiStorage
 from neuro_flow.types import LocalPath
 
 from ..parser import parse_bake_meta
+from ..storage.base import Storage
 from .root import Root
 
 
-if sys.version_info >= (3, 7):
-    from contextlib import AsyncExitStack
-else:
-    from async_exit_stack import AsyncExitStack
-
-
 @click.command()
 @option("--local-executor", is_flag=True, default=False, help="Run primary job locally")
 @click.option(
-    "--param", type=(str, str), multiple=True, help="Set params of the batch config"
+    "-p",
+    "--param",
+    type=(str, str),
+    multiple=True,
+    help="Set params of the batch config",
 )
 @click.option(
     "-n",
@@ -72,9 +65,7 @@ async def bake(
     """
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             BatchRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -92,11 +83,11 @@ async def bake(
 
 
 @click.command(hidden=True)
-@click.argument("executor_data")
+@click.argument("bake_id")
 @wrap_async()
 async def execute(
     root: Root,
-    executor_data: str,
+    bake_id: str,
 ) -> None:
     """Start a batch.
 
@@ -117,15 +108,12 @@ async def execute(
         # ignore everything, use neuro-flow cancel to stop the master job.
         signal.signal(signame, signal.SIG_IGN)
     async with AsyncExitStack() as stack:
-        data = ExecutorData.parse(executor_data)
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             BatchRunner(root.config_dir, root.console, client, storage, root)
         )
-        await runner.process(data)
+        await runner.process(bake_id)
 
 
 @click.command()
@@ -168,9 +156,7 @@ async def bakes(
     """List existing bakes."""
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             BatchRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -231,9 +217,7 @@ async def inspect(
     """
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             BatchRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -255,7 +239,15 @@ async def inspect(
 
 @click.command()
 @argument("bake", type=BAKE)
-@argument("task_id", type=FINISHED_TASK_AFTER_BAKE)
+@argument(
+    "task_id",
+    type=BakeTaskType(
+        include_started=False,
+        include_finished=True,
+        bake_id_param_name="bake",
+        attempt_no_param_name="attempt",
+    ),
+)
 @click.option(
     "-a",
     "--attempt",
@@ -286,9 +278,7 @@ async def show(
     """
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             BatchRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -317,9 +307,7 @@ async def cancel(
     """
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             BatchRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -346,9 +334,7 @@ async def clear_cache(
     """
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             BatchRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -388,9 +374,7 @@ async def restart(
     """
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             BatchRunner(root.config_dir, root.console, client, storage, root)
         )
