@@ -1,18 +1,13 @@
 import click
 import neuro_sdk
-import sys
+from contextlib import AsyncExitStack
 
 from neuro_flow.cli.click_types import LIVE_IMAGE_OR_ALL
 from neuro_flow.cli.root import Root
 from neuro_flow.cli.utils import argument, option, wrap_async
 from neuro_flow.live_runner import LiveRunner
-from neuro_flow.storage import APIStorage, NeuroStorageFS, Storage
-
-
-if sys.version_info >= (3, 7):
-    from contextlib import AsyncExitStack
-else:
-    from async_exit_stack import AsyncExitStack
+from neuro_flow.storage.api import ApiStorage
+from neuro_flow.storage.base import Storage
 
 
 @click.command()
@@ -32,9 +27,7 @@ async def build(root: Root, force_overwrite: bool, image: str) -> None:
     """
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             LiveRunner(root.config_dir, root.console, client, storage, root)
         )
