@@ -2,12 +2,13 @@ import click
 import logging
 import sys
 from click.exceptions import Abort as ClickAbort, Exit as ClickExit
+from neuro_cli.asyncio_utils import setup_child_watcher
 from neuro_cli.log_formatter import ConsoleHandler
 from rich.console import Console
 from typing import Any, List, Optional
 
 import neuro_flow
-from neuro_flow.cli import batch, completion, images, live, storage
+from neuro_flow.cli import batch, completion, file_logging, images, live, storage
 from neuro_flow.parser import ConfigDir, find_workspace
 from neuro_flow.types import LocalPath, TaskStatus
 
@@ -18,20 +19,20 @@ from .root import Root
 
 log = logging.getLogger(__name__)
 
-
-LOG_ERROR = log.error
+setup_child_watcher()
 
 
-def setup_logging(color: bool, verbosity: int) -> None:
+def setup_logging(color: bool, verbosity: int, show_traceback: bool) -> None:
     root_logger = logging.getLogger()
-    handler = ConsoleHandler(color)
-    root_logger.addHandler(handler)
+    console_handler = ConsoleHandler(color=color, show_traceback=show_traceback)
+    file_handler = file_logging.get_handler()
+    root_logger.addHandler(console_handler)
+    root_logger.addHandler(file_handler)
     root_logger.setLevel(logging.DEBUG)
 
-    if verbosity <= 1:
-        formatter = logging.Formatter()
-    else:
+    if verbosity > 1:
         formatter = logging.Formatter("%(name)s.%(funcName)s: %(message)s")
+        console_handler.setFormatter(formatter)
 
     if verbosity < -1:
         loglevel = logging.CRITICAL
@@ -44,8 +45,7 @@ def setup_logging(color: bool, verbosity: int) -> None:
     else:
         loglevel = logging.DEBUG
 
-    handler.setFormatter(formatter)
-    handler.setLevel(loglevel)
+    console_handler.setLevel(loglevel)
 
 
 class MainGroup(click.Group):
@@ -69,11 +69,11 @@ class MainGroup(click.Group):
         console = Console(highlight=False, log_path=False)
 
         verbosity = verbose - quiet
-        setup_logging(color=bool(console.color_system), verbosity=verbosity)
-
-        global LOG_ERROR
-        if show_traceback:
-            LOG_ERROR = log.exception
+        setup_logging(
+            color=bool(console.color_system),
+            verbosity=verbosity,
+            show_traceback=show_traceback,
+        )
 
         ctx.obj = Root(
             config_dir=config_dir,
@@ -163,6 +163,7 @@ cli.add_command(live.ps)
 cli.add_command(live.logs)
 cli.add_command(live.status)
 cli.add_command(live.kill)
+cli.add_command(live.delete_project)
 
 # Batch commands
 cli.add_command(batch.bake)
@@ -191,7 +192,7 @@ def main(args: Optional[List[str]] = None) -> None:
     try:
         cli.main(args=args, standalone_mode=False)
     except ClickAbort:
-        LOG_ERROR("Aborting.")
+        log.exception("Aborting.")
         sys.exit(130)
     except click.ClickException as e:
         e.show()
@@ -208,9 +209,9 @@ def main(args: Optional[List[str]] = None) -> None:
 
     except MultiError as e:
         for error in e.errors:
-            LOG_ERROR(f"{error}")
+            log.exception(f"{error}")
         sys.exit(1)
 
     except Exception as e:
-        LOG_ERROR(f"{e}")
+        log.exception(f"{e}")
         sys.exit(1)
