@@ -6,14 +6,14 @@ import enum
 import hashlib
 import itertools
 import json
+import logging
 import re
 import shlex
-import sys
 from abc import ABC, abstractmethod
+from contextlib import asynccontextmanager
 from datetime import timedelta
 from functools import lru_cache
 from neuro_sdk import Client
-from neuro_sdk.url_utils import uri_from_cli
 from typing import (
     AbstractSet,
     Any,
@@ -36,6 +36,7 @@ from typing_extensions import Annotated, Protocol
 from yarl import URL
 
 from neuro_flow import ast
+from neuro_flow.ast import InputType
 from neuro_flow.colored_topo_sorter import ColoredTopoSorter
 from neuro_flow.config_loader import ActionSpec, ConfigLoader
 from neuro_flow.expr import (
@@ -53,13 +54,12 @@ from neuro_flow.expr import (
     StrExpr,
     TypeT,
 )
-from neuro_flow.types import AlwaysT, FullID, LocalPath, RemotePath, TaskStatus
+from neuro_flow.types import AlwaysT, FullID, GitInfo, LocalPath, RemotePath, TaskStatus
+from neuro_flow.utils import collect_git_info
 
 
-if sys.version_info >= (3, 7):  # pragma: no cover
-    from contextlib import asynccontextmanager
-else:
-    from async_generator import asynccontextmanager
+log = logging.getLogger(__name__)
+
 
 # Exceptions
 
@@ -84,7 +84,7 @@ EnvCtx = Annotated[Mapping[str, str], "EnvCtx"]
 TagsCtx = Annotated[AbstractSet[str], "TagsCtx"]
 VolumesCtx = Annotated[Mapping[str, "VolumeCtx"], "VolumesCtx"]
 ImagesCtx = Annotated[Mapping[str, "ImageCtx"], "ImagesCtx"]
-InputsCtx = Annotated[Mapping[str, str], "InputsCtx"]
+InputsCtx = Annotated[Mapping[str, Union[int, float, bool, str]], "InputsCtx"]
 ParamsCtx = Annotated[Mapping[str, str], "ParamsCtx"]
 
 NeedsCtx = Annotated[Mapping[str, "DepCtx"], "NeedsCtx"]
@@ -210,6 +210,28 @@ class DepCtx:
         ), "CACHED status should replaced with SUCCEEDED for expressions"
 
 
+@dataclass(frozen=True)
+class GitCtx:
+    _git_info: Optional[GitInfo]
+
+    def _get_info(self) -> GitInfo:
+        if not self._git_info:
+            raise ValueError("Git info is not available: is this project under git?")
+        return self._git_info
+
+    @property
+    def sha(self) -> str:
+        return self._get_info().sha
+
+    @property
+    def branch(self) -> str:
+        return self._get_info().branch
+
+    @property
+    def tags(self) -> Sequence[str]:
+        return self._get_info().tags
+
+
 # Confs (similar to ..Ctx, but not available to expressions, only used
 # during evaluation)
 
@@ -304,6 +326,7 @@ class LocallyPreparedInfo:
 
     children_info: Mapping[str, "LocallyPreparedInfo"]
 
+    git_info: Optional[GitInfo]
     early_images: Mapping[str, EarlyImageCtx]
 
 
@@ -320,6 +343,10 @@ class EmptyRoot(RootABC):
         raise RuntimeError("neuro API is not available in <empty> context")
         yield Client()  # fake lint to make the code a real async iterator
 
+    @property
+    def dry_run(self) -> bool:
+        return False
+
 
 EMPTY_ROOT = EmptyRoot()
 
@@ -327,6 +354,7 @@ EMPTY_ROOT = EmptyRoot()
 @dataclass(frozen=True)
 class Context(RootABC):
     _client: Client
+    _dry_run: bool
 
     def lookup(self, name: str) -> TypeT:
         for f in fields(self):
@@ -343,6 +371,10 @@ class Context(RootABC):
     async def client(self) -> AsyncIterator[Client]:
         yield self._client
 
+    @property
+    def dry_run(self) -> bool:
+        return self._dry_run
+
 
 _MODULE_PARENT = TypeVar("_MODULE_PARENT", bound=RootABC, covariant=True)
 
@@ -371,17 +403,21 @@ class WithEnvContext(Context):
 
 @dataclass(frozen=True)
 class LiveContextStep1(WithFlowContext, Context):
+    git: GitCtx
+
     def to_live_ctx(
         self, env: EnvCtx, tags: TagsCtx, volumes: VolumesCtx, images: ImagesCtx
     ) -> "LiveContext":
         return LiveContext(
             project=self.project,
             flow=self.flow,
+            git=self.git,
             env=env,
             tags=tags,
             volumes=volumes,
             images=images,
             _client=self._client,
+            _dry_run=self._dry_run,
         )
 
 
@@ -395,12 +431,14 @@ class LiveContext(WithEnvContext, LiveContextStep1):
         return LiveJobContext(
             project=self.project,
             flow=self.flow,
+            git=self.git,
             env=self.env,
             tags=self.tags,
             volumes=self.volumes,
             images=self.images,
             params=params,
             _client=self._client,
+            _dry_run=self._dry_run,
         )
 
     def to_multi_job_ctx(
@@ -409,6 +447,7 @@ class LiveContext(WithEnvContext, LiveContextStep1):
         return LiveMultiJobContext(
             project=self.project,
             flow=self.flow,
+            git=self.git,
             env=self.env,
             tags=self.tags,
             volumes=self.volumes,
@@ -416,6 +455,7 @@ class LiveContext(WithEnvContext, LiveContextStep1):
             multi=multi,
             params=params,
             _client=self._client,
+            _dry_run=self._dry_run,
         )
 
 
@@ -444,6 +484,7 @@ class LiveModuleContext(ModuleContext[_MODULE_PARENT]):
 class BatchContextStep1(WithFlowContext, Context):
     flow: BatchFlowCtx
     params: ParamsCtx
+    git: GitCtx
 
     def to_step_2(
         self, env: EnvCtx, tags: TagsCtx, volumes: VolumesCtx, images: ImagesCtx
@@ -452,11 +493,13 @@ class BatchContextStep1(WithFlowContext, Context):
             project=self.project,
             flow=self.flow,
             params=self.params,
+            git=self.git,
             env=env,
             tags=tags,
             volumes=volumes,
             images=images,
             _client=self._client,
+            _dry_run=self._dry_run,
         )
 
 
@@ -474,12 +517,14 @@ class BatchContextStep2(WithEnvContext, BatchContextStep1):
             project=self.project,
             flow=self.flow,
             params=self.params,
+            git=self.git,
             env=self.env,
             tags=self.tags,
             volumes=self.volumes,
             images=self.images,
             strategy=strategy,
             _client=self._client,
+            _dry_run=self._dry_run,
         )
 
 
@@ -524,6 +569,7 @@ class BatchContext(BaseBatchContext, BatchContextStep2):
             project=self.project,
             flow=self.flow,
             params=self.params,
+            git=self.git,
             env=self.env,
             tags=self.tags,
             volumes=self.volumes,
@@ -531,6 +577,7 @@ class BatchContext(BaseBatchContext, BatchContextStep2):
             strategy=strategy,
             matrix=matrix,
             _client=self._client,
+            _dry_run=self._dry_run,
         )
 
 
@@ -543,6 +590,7 @@ class BatchMatrixContext(BaseMatrixContext, BatchContext):
             project=self.project,
             flow=self.flow,
             params=self.params,
+            git=self.git,
             env=self.env,
             tags=self.tags,
             volumes=self.volumes,
@@ -552,6 +600,7 @@ class BatchMatrixContext(BaseMatrixContext, BatchContext):
             needs=needs,
             state=state,
             _client=self._client,
+            _dry_run=self._dry_run,
         )
 
 
@@ -565,13 +614,16 @@ class BatchTaskContext(BaseTaskContext, BatchMatrixContext):
 class BatchActionContextStep1(ModuleContext[_MODULE_PARENT]):
     inputs: InputsCtx
     strategy: StrategyCtx
+    git: GitCtx
 
     def to_action_ctx(self, images: ImagesCtx) -> "BatchActionContext[_MODULE_PARENT]":
         return BatchActionContext(
             inputs=self.inputs,
             strategy=self.strategy,
+            git=self.git,
             images=images,
             _client=self._client,
+            _dry_run=self._dry_run,
             _parent=self._parent,
         )
 
@@ -586,9 +638,11 @@ class BatchActionContext(BatchActionContextStep1[_MODULE_PARENT], BaseBatchConte
         return BatchActionMatrixContext(
             inputs=self.inputs,
             images=self.images,
+            git=self.git,
             matrix=matrix,
             strategy=strategy,
             _client=self._client,
+            _dry_run=self._dry_run,
             _parent=self._parent,
         )
 
@@ -599,8 +653,10 @@ class BatchActionContext(BatchActionContextStep1[_MODULE_PARENT], BaseBatchConte
             strategy=self.strategy,
             inputs=self.inputs,
             images=self.images,
+            git=self.git,
             needs=needs,
             _client=self._client,
+            _dry_run=self._dry_run,
             _parent=self._parent,
         )
 
@@ -623,9 +679,11 @@ class BatchActionMatrixContext(BaseMatrixContext, BatchActionContext[_MODULE_PAR
             matrix=self.matrix,
             strategy=self.strategy,
             images=self.images,
+            git=self.git,
             needs=needs,
             state=state,
             _client=self._client,
+            _dry_run=self._dry_run,
             _parent=self._parent,
         )
 
@@ -810,13 +868,11 @@ async def setup_local_or_storage_path(
     if path_str is None:
         return None
     async with ctx.client() as client:
-        username = client.config.username
-        cluster_name = client.config.cluster_name
-    if path_str.startswith("storage"):
-        try:
-            return uri_from_cli(path_str, username, cluster_name)
-        except ValueError as e:
-            raise EvalError(str(e), str_expr.start, str_expr.end)
+        if path_str.startswith("storage"):
+            try:
+                return client.parse.str_to_uri(path_str)
+            except ValueError as e:
+                raise EvalError(str(e), str_expr.start, str_expr.end)
     try:
         path = LocalPath(path_str)
     except ValueError as e:
@@ -951,8 +1007,10 @@ async def validate_action_call(
     call_ast: Union[ast.BaseActionCall, ast.BaseModuleCall],
     ast_inputs: Optional[Mapping[str, ast.Input]],
 ) -> None:
+    supported_inputs: Set[str]
+    supplied_inputs: Set[str]
     if ast_inputs:
-        supported_inputs = ast_inputs.keys()
+        supported_inputs = set(ast_inputs.keys())
         required_inputs = {
             input_name
             for input_name, input_ast in ast_inputs.items()
@@ -962,7 +1020,7 @@ async def validate_action_call(
         supported_inputs = set()
         required_inputs = set()
     if call_ast.args:
-        supplied_inputs = call_ast.args.keys()
+        supplied_inputs = set(call_ast.args.keys())
     else:
         supplied_inputs = set()
     missing = required_inputs - supplied_inputs
@@ -983,15 +1041,34 @@ async def validate_action_call(
 
 async def setup_inputs_ctx(
     ctx: RootABC,
-    call_ast: Union[
-        ast.BaseActionCall, ast.BaseModuleCall
-    ],  # Only used to generate errors
+    call_ast: Union[ast.BaseActionCall, ast.BaseModuleCall],
     ast_inputs: Optional[Mapping[str, ast.Input]],
 ) -> InputsCtx:
     await validate_action_call(call_ast, ast_inputs)
     if call_ast.args is None or ast_inputs is None:
         return {}
     inputs = {k: await v.eval(ctx) for k, v in call_ast.args.items()}
+    for key, value in inputs.copy().items():
+        input_ast = ast_inputs[key]
+        arg_ast = call_ast.args[key]
+        if input_ast.type == InputType.STR:
+            if not isinstance(value, str):
+                eval_error = EvalError(
+                    f"Implicit casting of action argument '{key}' to string"
+                    f" is deprecated",
+                    arg_ast.start,
+                    arg_ast.end,
+                )
+                log.warning(str(eval_error))
+            inputs[key] = str(value)
+        elif not isinstance(value, input_ast.type.to_type()):
+            raise EvalError(
+                f"Type of argument '{key}' do not match to with inputs declared "
+                f"type. Argument has type '{type(value).__name__}', declared "
+                f"input type is '{input_ast.type.value}'",
+                arg_ast.start,
+                arg_ast.end,
+            )
     for name, inp in ast_inputs.items():
         if name not in inputs and inp.default.pattern is not None:
             val = await inp.default.eval(EMPTY_ROOT)
@@ -1050,14 +1127,19 @@ async def setup_strategy_ctx(
     return StrategyCtx(fail_fast=fail_fast, max_parallel=max_parallel)
 
 
-async def setup_matrix(ast_matrix: Optional[ast.Matrix]) -> Sequence[MatrixCtx]:
+async def setup_matrix(
+    ctx: RootABC,
+    ast_matrix: Optional[ast.Matrix],
+) -> Sequence[MatrixCtx]:
     if ast_matrix is None:
         return [{}]
     # Init
     products = []
     for k, lst in ast_matrix.products.items():
-        lst2 = [{k: await i.eval(EMPTY_ROOT)} for i in lst]
-        products.append(lst2)
+        values = await lst.eval(ctx)
+        if values:
+            lst2 = [{k: v} for v in values]
+            products.append(lst2)
     matrices = []
     for row in itertools.product(*products):
         dct: Dict[str, LiteralT] = {}
@@ -1067,7 +1149,7 @@ async def setup_matrix(ast_matrix: Optional[ast.Matrix]) -> Sequence[MatrixCtx]:
     # Exclude
     exclude = []
     for exc_spec in ast_matrix.exclude:
-        exclude.append({k: await v.eval(EMPTY_ROOT) for k, v in exc_spec.items()})
+        exclude.append({k: await v.eval(ctx) for k, v in exc_spec.items()})
     filtered = []
     for matrix in matrices:
         include = True
@@ -1101,7 +1183,9 @@ async def setup_matrix(ast_matrix: Optional[ast.Matrix]) -> Sequence[MatrixCtx]:
                 ast_matrix._start,
                 ast_matrix._end,
             )
-        matrices.append({k: await v.eval(EMPTY_ROOT) for k, v in inc_spec.items()})
+        matrices.append({k: await v.eval(ctx) for k, v in inc_spec.items()})
+    for pos, dct in enumerate(matrices):
+        dct["ORDINAL"] = pos
     return matrices
 
 
@@ -1149,7 +1233,7 @@ _MergeTarget = TypeVar("_MergeTarget", bound=SupportsAstMerge)
 async def merge_asts(child: _MergeTarget, parent: SupportsAstMerge) -> _MergeTarget:
     child_fields = {f.name for f in dataclasses.fields(child)}
     for field in parent._specified_fields:
-        if field == "inherits" or field not in child_fields:
+        if field == "mixins" or field not in child_fields:
             continue
         field_present = field in child._specified_fields
         child_value = getattr(child, field)
@@ -1178,7 +1262,7 @@ async def merge_asts(child: _MergeTarget, parent: SupportsAstMerge) -> _MergeTar
 
 class MixinApplyTarget(Protocol):
     @property
-    def inherits(self) -> Optional[Sequence[StrExpr]]:
+    def mixins(self) -> Optional[Sequence[StrExpr]]:
         ...
 
     @property
@@ -1192,9 +1276,9 @@ _MixinApplyTarget = TypeVar("_MixinApplyTarget", bound=MixinApplyTarget)
 async def apply_mixins(
     base: _MixinApplyTarget, mixins: Mapping[str, SupportsAstMerge]
 ) -> _MixinApplyTarget:
-    if base.inherits is None:
+    if base.mixins is None:
         return base
-    for mixin_expr in reversed(base.inherits):
+    for mixin_expr in reversed(base.mixins):
         mixin_name = await mixin_expr.eval(EMPTY_ROOT)
         try:
             mixin = mixins[mixin_name]
@@ -1215,10 +1299,8 @@ async def setup_mixins(
         return {}
     graph: Dict[str, Dict[str, int]] = {}
     for mixin_name, mixin in raw_mixins.items():
-        inherits = mixin.inherits or []
-        graph[mixin_name] = {
-            await dep_expr.eval(EMPTY_ROOT): 1 for dep_expr in inherits
-        }
+        mixins = mixin.mixins or []
+        graph[mixin_name] = {await dep_expr.eval(EMPTY_ROOT): 1 for dep_expr in mixins}
     topo = ColoredTopoSorter(graph)
     result: Dict[str, _MixinApplyTarget] = {}
     while not topo.is_all_colored(1):
@@ -1232,7 +1314,7 @@ class RunningLiveFlow:
     _ast_flow: ast.LiveFlow
     _ctx: LiveContext
     _cl: ConfigLoader
-    _mixins: Optional[Mapping[str, ast.JobMixin]] = None
+    _mixins: Mapping[str, SupportsAstMerge]
 
     def __init__(
         self,
@@ -1240,11 +1322,13 @@ class RunningLiveFlow:
         ctx: LiveContext,
         config_loader: ConfigLoader,
         defaults: DefaultsConf,
+        mixins: Mapping[str, SupportsAstMerge],
     ):
         self._ast_flow = ast_flow
         self._ctx = ctx
         self._cl = config_loader
         self._defaults = defaults
+        self._mixins = mixins
 
     @property
     def job_ids(self) -> Iterable[str]:
@@ -1274,18 +1358,13 @@ class RunningLiveFlow:
         # Simple shortcut
         return (await self.get_meta(job_id)).multi
 
-    async def get_mixins(self) -> Mapping[str, ast.JobMixin]:
-        if self._mixins is None:
-            self._mixins = await setup_mixins(self._ast_flow.mixins)
-        return self._mixins
-
     async def _get_job_ast(
         self, job_id: str
     ) -> Union[ast.Job, ast.JobActionCall, ast.JobModuleCall]:
         try:
             base = self._ast_flow.jobs[job_id]
             if isinstance(base, ast.Job):
-                base = await apply_mixins(base, await self.get_mixins())
+                base = await apply_mixins(base, self._mixins)
             return base
         except KeyError:
             raise UnknownJob(job_id)
@@ -1370,6 +1449,7 @@ class RunningLiveFlow:
             ctx = LiveActionContext(
                 inputs=await setup_inputs_ctx(ctx, job, action_ast.inputs),
                 _client=self._ctx._client,
+                _dry_run=self._ctx._dry_run,
             )
             env_ctx = {}
             defaults = DefaultsConf()
@@ -1380,6 +1460,7 @@ class RunningLiveFlow:
                 inputs=await setup_inputs_ctx(ctx, job, action_ast.inputs),
                 _parent=ctx,
                 _client=self._ctx._client,
+                _dry_run=self._ctx._dry_run,
             )
             job = action_ast.job
         assert isinstance(job, ast.Job)
@@ -1454,7 +1535,10 @@ class RunningLiveFlow:
 
     @classmethod
     async def create(
-        cls, config_loader: ConfigLoader, config_name: str = "live"
+        cls,
+        config_loader: ConfigLoader,
+        config_name: str = "live",
+        dry_run: bool = False,
     ) -> "RunningLiveFlow":
         ast_flow = await config_loader.fetch_flow(config_name)
         ast_project = await config_loader.fetch_project()
@@ -1465,9 +1549,14 @@ class RunningLiveFlow:
         flow_ctx = await setup_flow_ctx(
             EMPTY_ROOT, ast_flow, config_name, config_loader, project_ctx
         )
+        git_ctx = GitCtx(await collect_git_info())
 
         step_1_ctx = LiveContextStep1(
-            project=project_ctx, flow=flow_ctx, _client=config_loader.client
+            project=project_ctx,
+            flow=flow_ctx,
+            git=git_ctx,
+            _client=config_loader.client,
+            _dry_run=dry_run,
         )
 
         defaults, env, tags = await setup_defaults_env_tags_ctx(
@@ -1491,7 +1580,13 @@ class RunningLiveFlow:
             images=images,
         )
 
-        return cls(ast_flow, live_ctx, config_loader, defaults)
+        raw_mixins: Mapping[str, MixinApplyTarget] = {
+            **(ast_project.mixins or {}),
+            **(ast_flow.mixins or {}),
+        }
+        mixins = await setup_mixins(raw_mixins)
+
+        return cls(ast_flow, live_ctx, config_loader, defaults, mixins)
 
 
 _T = TypeVar("_T", bound=BaseBatchContext, covariant=True)
@@ -1514,7 +1609,7 @@ class EarlyBatch:
 
     @property
     @abstractmethod
-    def mixins(self) -> Optional[Mapping[str, ast.TaskMixin]]:
+    def mixins(self) -> Optional[Mapping[str, SupportsAstMerge]]:
         pass
 
     @property
@@ -1613,7 +1708,7 @@ class EarlyBatch:
             mixins = None
 
         tasks = await EarlyTaskGraphBuilder(
-            self._cl, prep_task.action.tasks, mixins
+            self._flow_ctx, self._cl, prep_task.action.tasks, mixins
         ).build()
         early_images = await setup_images_early(
             self._flow_ctx, self._flow_ctx, prep_task.action.images
@@ -1647,7 +1742,7 @@ class EarlyBatchAction(EarlyBatch):
         config_loader: ConfigLoader,
         action: ast.BatchAction,
         parent_ctx_class: Type[RootABC],
-        mixins: Optional[Mapping[str, ast.TaskMixin]],
+        mixins: Optional[Mapping[str, SupportsAstMerge]],
     ):
         super().__init__(ctx, tasks, config_loader)
         self._action = action
@@ -1660,7 +1755,7 @@ class EarlyBatchAction(EarlyBatch):
         return self._early_images
 
     @property
-    def mixins(self) -> Optional[Mapping[str, ast.TaskMixin]]:
+    def mixins(self) -> Optional[Mapping[str, SupportsAstMerge]]:
         return self._mixins
 
     def get_image_ast(self, image_id: str) -> ast.Image:
@@ -1790,6 +1885,7 @@ class RunningBatchBase(Generic[_T], EarlyBatch, ABC):
                     ctx, prep_task.call, prep_task.action.inputs
                 ),
                 _client=self._ctx._client,
+                _dry_run=self._ctx._dry_run,
             )
             defaults = DefaultsConf()  # TODO: Is it correct?
 
@@ -1923,6 +2019,7 @@ class RunningBatchBase(Generic[_T], EarlyBatch, ABC):
         action_ctx = LocalActionContext(
             inputs=await setup_inputs_ctx(ctx, prep_task.call, prep_task.action.inputs),
             _client=self._ctx._client,
+            _dry_run=self._ctx._dry_run,
         )
 
         return LocalTask(
@@ -1941,7 +2038,8 @@ class RunningBatchFlow(RunningBatchBase[BatchContext]):
         bake_id: str,
         local_info: Optional[LocallyPreparedInfo],
         ast_flow: ast.BatchFlow,
-        mixins: Optional[Mapping[str, ast.TaskMixin]],
+        ast_project: ast.Project,
+        mixins: Optional[Mapping[str, SupportsAstMerge]],
     ):
         super().__init__(
             ctx,
@@ -1954,17 +2052,27 @@ class RunningBatchFlow(RunningBatchBase[BatchContext]):
             local_info,
         )
         self._ast_flow = ast_flow
+        self._ast_project = ast_project
         self._mixins = mixins
 
     def get_image_ast(self, image_id: str) -> ast.Image:
-        if self._ast_flow.images is None:
-            raise KeyError(image_id)
-        return self._ast_flow.images[image_id]
+        try:
+            if self._ast_flow.images is None:
+                raise KeyError(image_id)
+            return self._ast_flow.images[image_id]
+        except KeyError:
+            if self._ast_project.images is not None:
+                return self._ast_project.images[image_id]
+            raise
 
     @property
-    def mixins(self) -> Optional[Mapping[str, ast.TaskMixin]]:
+    def mixins(self) -> Optional[Mapping[str, SupportsAstMerge]]:
         return self._mixins
 
+    @property
+    def params(self) -> Mapping[str, str]:
+        return self._ctx.params
+
     @property
     def project_id(self) -> str:
         return self._ctx.flow.project_id
@@ -1991,6 +2099,7 @@ class RunningBatchFlow(RunningBatchBase[BatchContext]):
         bake_id: str,
         params: Optional[Mapping[str, str]] = None,
         local_info: Optional[LocallyPreparedInfo] = None,
+        dry_run: bool = False,
     ) -> "RunningBatchFlow":
         ast_flow = await config_loader.fetch_flow(batch)
         ast_project = await config_loader.fetch_project()
@@ -2007,7 +2116,9 @@ class RunningBatchFlow(RunningBatchBase[BatchContext]):
             project=project_ctx,
             flow=flow_ctx,
             params=params_ctx,
+            git=GitCtx(local_info.git_info if local_info else None),
             _client=config_loader.client,
+            _dry_run=dry_run,
         )
         if local_info is None:
             early_images: Mapping[str, EarlyImageCtx] = {
@@ -2072,7 +2183,11 @@ class RunningBatchFlow(RunningBatchBase[BatchContext]):
             ),
         )
 
-        mixins = await setup_mixins(ast_flow.mixins)
+        raw_mixins: Mapping[str, MixinApplyTarget] = {
+            **(ast_project.mixins or {}),
+            **(ast_flow.mixins or {}),
+        }
+        mixins = await setup_mixins(raw_mixins)
         tasks = await TaskGraphBuilder(
             batch_ctx, config_loader, cache_conf, ast_flow.tasks, mixins
         ).build()
@@ -2085,6 +2200,7 @@ class RunningBatchFlow(RunningBatchBase[BatchContext]):
             bake_id,
             local_info,
             ast_flow,
+            ast_project,
             mixins,
         )
 
@@ -2101,7 +2217,7 @@ class RunningBatchActionFlow(RunningBatchBase[BatchActionContext[RootABC]]):
         action: ast.BatchAction,
         bake_id: str,
         local_info: Optional[LocallyPreparedInfo],
-        mixins: Optional[Mapping[str, ast.TaskMixin]],
+        mixins: Optional[Mapping[str, SupportsAstMerge]],
     ):
         super().__init__(
             flow_ctx,
@@ -2122,7 +2238,7 @@ class RunningBatchActionFlow(RunningBatchBase[BatchActionContext[RootABC]]):
         return self._action.images[image_id]
 
     @property
-    def mixins(self) -> Optional[Mapping[str, ast.TaskMixin]]:
+    def mixins(self) -> Optional[Mapping[str, SupportsAstMerge]]:
         return self._mixins
 
     async def calc_outputs(self, task_results: NeedsCtx) -> DepCtx:
@@ -2154,13 +2270,15 @@ class RunningBatchActionFlow(RunningBatchBase[BatchActionContext[RootABC]]):
         bake_id: str,
         local_info: Optional[LocallyPreparedInfo],
         defaults: DefaultsConf = DefaultsConf(),
-        mixins: Optional[Mapping[str, ast.TaskMixin]] = None,
+        mixins: Optional[Mapping[str, SupportsAstMerge]] = None,
     ) -> "RunningBatchActionFlow":
         step_1_ctx = BatchActionContextStep1(
             inputs=inputs,
             strategy=base_strategy,
+            git=GitCtx(local_info.git_info if local_info else None),
             _client=config_loader.client,
             _parent=parent_ctx,
+            _dry_run=parent_ctx.dry_run,
         )
 
         if local_info is None:
@@ -2493,10 +2611,12 @@ class EarlyTaskGraphBuilder:
 
     def __init__(
         self,
+        ctx: RootABC,
         config_loader: ConfigLoader,
         ast_tasks: Sequence[Union[ast.Task, ast.TaskActionCall, ast.TaskModuleCall]],
-        mixins: Optional[Mapping[str, ast.TaskMixin]],
+        mixins: Optional[Mapping[str, SupportsAstMerge]],
     ):
+        self._ctx = ctx
         self._cl = config_loader
         self._ast_tasks = ast_tasks
         self._mixins = mixins or {}
@@ -2522,7 +2642,7 @@ class EarlyTaskGraphBuilder:
             )
 
             matrix_ast = ast_task.strategy.matrix if ast_task.strategy else None
-            matrices = await setup_matrix(matrix_ast)
+            matrices = await setup_matrix(self._ctx, matrix_ast)
 
             if len(matrices) > self.MATRIX_SIZE_LIMIT:
                 assert matrix_ast
@@ -2539,6 +2659,7 @@ class EarlyTaskGraphBuilder:
                 matrix_ctx = MatrixOnlyContext(
                     matrix=matrix,
                     _client=self._cl.client,
+                    _dry_run=False,
                 )
 
                 task_id, real_id = await self._setup_ids(matrix_ctx, num, ast_task)
@@ -2659,8 +2780,10 @@ class EarlyTaskGraphBuilder:
         task_id = await ast_task.id.eval(ctx)
         if task_id is None:
             # Dash is not allowed in identifier, so the generated read id
-            # never clamps with user_provided one.
-            suffix = [str(ctx.matrix[k]) for k in sorted(ctx.matrix)]
+            # never clamps with user-provided one.
+            # Filter system properties
+            keys = [key for key in sorted(ctx.matrix) if key == key.lower()]
+            suffix = [str(ctx.matrix[key]) for key in keys]
             real_id = "-".join(["task", str(num), *suffix])
         else:
             real_id = task_id
@@ -2681,6 +2804,7 @@ class EarlyTaskGraphBuilder:
 
 class TaskGraphBuilder(EarlyTaskGraphBuilder):
     MATRIX_SIZE_LIMIT = 256
+    _ctx: BaseBatchContext
 
     def __init__(
         self,
@@ -2688,9 +2812,9 @@ class TaskGraphBuilder(EarlyTaskGraphBuilder):
         config_loader: ConfigLoader,
         default_cache: CacheConf,
         ast_tasks: Sequence[Union[ast.Task, ast.TaskActionCall, ast.TaskModuleCall]],
-        mixins: Optional[Mapping[str, ast.TaskMixin]],
+        mixins: Optional[Mapping[str, SupportsAstMerge]],
     ):
-        super().__init__(config_loader, ast_tasks, mixins)
+        super().__init__(ctx, config_loader, ast_tasks, mixins)
         self._ctx = ctx
         self._default_cache = default_cache
 
