@@ -8,6 +8,7 @@ import shlex
 import sys
 from neuro_sdk import (
     Action,
+    AuthorizationError,
     Client,
     IllegalArgumentError,
     JobDescription,
@@ -15,7 +16,6 @@ from neuro_sdk import (
     Permission,
     ResourceNotFound,
 )
-from neuro_sdk.storage import normalize_storage_path_uri  # type: ignore
 from rich import box
 from rich.console import Console
 from rich.table import Table
@@ -36,7 +36,7 @@ from yarl import URL
 from .config_loader import LiveLocalCL
 from .context import ImageCtx, JobMeta, RunningLiveFlow, UnknownJob, VolumeCtx
 from .parser import ConfigDir
-from .storage import Storage
+from .storage.base import ProjectStorage, Storage
 from .types import TaskStatus
 from .utils import (
     RUNNING_JOB_STATUSES,
@@ -66,6 +66,7 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
         client: Client,
         storage: Storage,
         global_options: GlobalOptions,
+        dry_run: bool = False,
     ) -> None:
         self._config_dir = config_dir
         self._console = console
@@ -73,28 +74,37 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
         self._flow: Optional[RunningLiveFlow] = None
         self._client = client
         self._storage = storage
+        self._project_storage: Optional[ProjectStorage] = None
         self._is_projet_role_created = False
         self._run_neuro_cli = make_cmd_exec(
             "neuro", global_options=encode_global_options(global_options)
         )
         self._run_extras_cli = make_cmd_exec("neuro-extras")
+        self._dry_run = dry_run
 
-    async def post_init(self) -> None:
+    async def init_flow(self) -> None:
         if self._flow is not None:
             return
         self._config_loader = LiveLocalCL(self._config_dir, self._client)
         try:
-            self._flow = await RunningLiveFlow.create(self._config_loader)
+            self._flow = await RunningLiveFlow.create(
+                self._config_loader, dry_run=self._dry_run
+            )
         except Exception:
             await self._config_loader.close()
             raise
 
+    async def init_project_storage(self) -> None:
+        project = await self._storage.get_or_create_project(self.flow.project.id)
+        self._project_storage = self._storage.project(id=project.id)
+
     async def close(self) -> None:
         if self._config_loader is not None:
             await self._config_loader.close()
 
     async def __aenter__(self) -> "LiveRunner":
-        await self.post_init()
+        await self.init_flow()
+        await self.init_project_storage()
         return self
 
     async def __aexit__(
@@ -110,6 +120,11 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
         assert self._flow is not None
         return self._flow
 
+    @property
+    def storage(self) -> ProjectStorage:
+        assert self._project_storage is not None
+        return self._project_storage
+
     @property
     def client(self) -> Client:
         assert self._client is not None
@@ -125,6 +140,7 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
                     if not skip_check:
                         raise click.BadArgumentUsage(
                             f"Please provide a suffix for multi-job {fmt_id(job_id)}"
+                            " via -s/--suffix option."
                         )
             else:
                 if suffix is not None:
@@ -337,7 +353,6 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
         suffix: Optional[str],
         args: Optional[Tuple[str]],
         params: Mapping[str, str],
-        dry_run: bool,
     ) -> None:
         """Run a named job"""
         assert self._flow is not None
@@ -350,7 +365,7 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
                 "Additional job arguments are supported by multi-jobs only"
             )
 
-        if not dry_run and await self._try_attach_to_running(
+        if not self._dry_run and await self._try_attach_to_running(
             job_id, suffix, args, params
         ):
             return  # Attached to running job
@@ -420,25 +435,31 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
             run_args.append(f"--share={project_role}")
 
         run_args.append(job.image)
+
+        run_args.extend(["--"])
         if job.cmd:
             run_args.extend(shlex.split(job.cmd))
 
         if job.multi and args:
             run_args.extend(args)
 
-        if dry_run:
+        if self._dry_run:
             run_args = ["neuro", *run_args]
             self._console.print(
                 " ".join(shlex.quote(arg) for arg in run_args), soft_wrap=True
             )
             return
 
-        jobs = []
+        live_job_metas = []
         for job_id in self.flow.job_ids:
             job_meta = await self.flow.get_meta(job_id)
-            jobs.append(job_meta)
-        await self._storage.ensure_project(self.flow.flow.project_id)
-        await self._storage.write_live(self.flow.flow.project_id, jobs)
+            live_job_metas.append(job_meta)
+        for job_meta in live_job_metas:
+            await self.storage.replace_live_job(
+                multi=job_meta.multi,
+                yaml_id=job_meta.id,
+                tags=job_meta.tags,
+            )
         await self._run_neuro_cli(*run_args)
 
     async def logs(self, job_id: str, suffix: Optional[str]) -> None:
@@ -539,8 +560,9 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
             str(volume_ctx.full_local_path),
             str(volume_ctx.remote),
         )
-        uri = normalize_storage_path_uri(
-            volume_ctx.remote, self._client.username, self._client.cluster_name
+        uri = self._client.parse.normalize_uri(
+            volume_ctx.remote,
+            allowed_schemes=("storage",),
         )
         await self._add_resource(uri)
 
@@ -584,8 +606,9 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
                     "--parents",
                     str(volume_ctx.remote),
                 )
-                uri = normalize_storage_path_uri(
-                    volume_ctx.remote, self._client.username, self._client.cluster_name
+                uri = self._client.parse.normalize_uri(
+                    volume_ctx.remote,
+                    allowed_schemes=("storage",),
                 )
                 await self._add_resource(uri)
 
@@ -640,14 +663,20 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
             return
         try:
             await self._client.users.add(project_role)
+        except AuthorizationError:
+            pass
+            # We have no permissions to create role --
+            # assume that this is shared project and
+            # current user is not the owner
         except IllegalArgumentError as e:
             if "already exists" not in str(e):
                 raise
         self._is_projet_role_created = True
 
     async def _add_storage_resource(self, uri: URL) -> None:
-        uri = normalize_storage_path_uri(
-            uri, self._client.username, self._client.cluster_name
+        uri = self._client.parse.normalize_uri(
+            uri,
+            allowed_schemes=("storage",),
         )
 
     async def _add_resource(self, uri: URL) -> None:
