@@ -68,7 +68,7 @@ class Tokenizer:
         ("BIN", r"0[bB][0-1_]+"),
         ("INT", r"-?[0-9][0-9_]*"),
         ("STR", r"'[^']*'|" r'"[^"]*"'),
-        ("OP", r"(==|!=|or|and|<=|<|>=|>|\||\+|-|\*|/)(?=[ \t])"),
+        ("OP", r"(==|!=|or|and|<=|<|>=|>|\||\+|-|\*|/|%)(?=[ \t])"),
         ("UNARY_OP", r"(not)(?=[ \t])"),
         ("NAME", r"[A-Za-z][A-Za-z_0-9]*"),
         ("DOT", r"\."),
@@ -113,11 +113,6 @@ class Tokenizer:
                 return None
             else:
                 idx = ltmpl.start()
-        rtmpl = self.RTMPL_RE.search(s, i, idx)
-        if rtmpl is not None:
-            t = self.make_token("TEXT", s[i : rtmpl.start()], pos)
-            errline = s.splitlines()[t.end.line - start.line]
-            raise LexerError(t.end, " " * start.col + errline)
         return self.make_token("TEXT", s[i:idx], pos)
 
     def __call__(self, s: str, start: Pos) -> Iterator[Token]:
