@@ -16,6 +16,7 @@ from neuro_flow.expr import (
     OptRemotePathExpr,
     OptStrExpr,
     OptTimeDeltaExpr,
+    PrimitiveExpr,
     RemotePathExpr,
     SequenceItemsExpr,
     SimpleOptIdExpr,
@@ -197,7 +198,7 @@ def test_parse_minimal(assets: pathlib.Path) -> None:
                     "id",
                     "volumes",
                 },
-                inherits=None,
+                mixins=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "test_a"),
                 title=OptStrExpr(
                     Pos(0, 0, config_file), Pos(0, 0, config_file), "Batch title"
@@ -325,7 +326,7 @@ def test_parse_seq(assets: pathlib.Path) -> None:
                 _start=Pos(2, 4, config_file),
                 _end=Pos(6, 2, config_file),
                 _specified_fields={"preset", "cmd", "image"},
-                inherits=None,
+                mixins=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs=None,
@@ -375,7 +376,7 @@ def test_parse_seq(assets: pathlib.Path) -> None:
                 _start=Pos(6, 4, config_file),
                 _end=Pos(9, 0, config_file),
                 _specified_fields={"preset", "cmd", "image"},
-                inherits=None,
+                mixins=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs=None,
@@ -456,7 +457,7 @@ def test_parse_needs(assets: pathlib.Path) -> None:
                 _start=Pos(2, 4, config_file),
                 _end=Pos(7, 2, config_file),
                 _specified_fields={"cmd", "image", "id", "preset"},
-                inherits=None,
+                mixins=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "task_a"),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs=None,
@@ -506,7 +507,7 @@ def test_parse_needs(assets: pathlib.Path) -> None:
                 _start=Pos(7, 4, config_file),
                 _end=Pos(11, 0, config_file),
                 _specified_fields={"needs", "image", "cmd", "preset"},
-                inherits=None,
+                mixins=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs={
@@ -591,7 +592,7 @@ def test_parse_needs_dict(assets: pathlib.Path) -> None:
                 _start=Pos(2, 4, config_file),
                 _end=Pos(7, 2, config_file),
                 _specified_fields={"preset", "image", "cmd", "id"},
-                inherits=None,
+                mixins=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "task_a"),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs=None,
@@ -641,7 +642,7 @@ def test_parse_needs_dict(assets: pathlib.Path) -> None:
                 _start=Pos(7, 4, config_file),
                 _end=Pos(12, 0, config_file),
                 _specified_fields={"preset", "image", "cmd", "needs"},
-                inherits=None,
+                mixins=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs={
@@ -728,7 +729,7 @@ def test_parse_matrix(assets: pathlib.Path) -> None:
                 _start=Pos(2, 4, config_file),
                 _end=Pos(14, 0, config_file),
                 _specified_fields={"strategy", "image", "cmd"},
-                inherits=None,
+                mixins=None,
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 image=OptStrExpr(
@@ -771,39 +772,51 @@ def test_parse_matrix(assets: pathlib.Path) -> None:
                         _start=Pos(4, 8, config_file),
                         _end=Pos(12, 4, config_file),
                         products={
-                            "one": [
-                                StrExpr(
-                                    Pos(0, 0, config_file), Pos(0, 0, config_file), "o1"
-                                ),
-                                StrExpr(
-                                    Pos(0, 0, config_file), Pos(0, 0, config_file), "o2"
-                                ),
-                            ],
-                            "two": [
-                                StrExpr(
-                                    Pos(0, 0, config_file), Pos(0, 0, config_file), "t1"
-                                ),
-                                StrExpr(
-                                    Pos(0, 0, config_file), Pos(0, 0, config_file), "t2"
-                                ),
-                            ],
+                            "one": SequenceItemsExpr(
+                                [
+                                    PrimitiveExpr(
+                                        Pos(0, 0, config_file),
+                                        Pos(0, 0, config_file),
+                                        "o1",
+                                    ),
+                                    PrimitiveExpr(
+                                        Pos(0, 0, config_file),
+                                        Pos(0, 0, config_file),
+                                        "o2",
+                                    ),
+                                ]
+                            ),
+                            "two": SequenceItemsExpr(
+                                [
+                                    PrimitiveExpr(
+                                        Pos(0, 0, config_file),
+                                        Pos(0, 0, config_file),
+                                        "t1",
+                                    ),
+                                    PrimitiveExpr(
+                                        Pos(0, 0, config_file),
+                                        Pos(0, 0, config_file),
+                                        "t2",
+                                    ),
+                                ]
+                            ),
                         },
                         exclude=[
                             {
-                                "one": StrExpr(
+                                "one": PrimitiveExpr(
                                     Pos(0, 0, config_file), Pos(0, 0, config_file), "o1"
                                 ),
-                                "two": StrExpr(
+                                "two": PrimitiveExpr(
                                     Pos(0, 0, config_file), Pos(0, 0, config_file), "t2"
                                 ),
                             }
                         ],
                         include=[
                             {
-                                "one": StrExpr(
+                                "one": PrimitiveExpr(
                                     Pos(0, 0, config_file), Pos(0, 0, config_file), "o3"
                                 ),
-                                "two": StrExpr(
+                                "two": PrimitiveExpr(
                                     Pos(0, 0, config_file), Pos(0, 0, config_file), "t3"
                                 ),
                             }
@@ -885,7 +898,7 @@ def test_parse_matrix_with_strategy(assets: pathlib.Path) -> None:
                 _start=Pos(8, 4, config_file),
                 _end=Pos(25, 2, config_file),
                 _specified_fields={"image", "strategy", "cmd", "cache"},
-                inherits=None,
+                mixins=None,
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 image=OptStrExpr(
@@ -928,39 +941,51 @@ def test_parse_matrix_with_strategy(assets: pathlib.Path) -> None:
                         Pos(10, 8, config_file),
                         Pos(18, 6, config_file),
                         products={
-                            "one": [
-                                StrExpr(
-                                    Pos(0, 0, config_file), Pos(0, 0, config_file), "o1"
-                                ),
-                                StrExpr(
-                                    Pos(0, 0, config_file), Pos(0, 0, config_file), "o2"
-                                ),
-                            ],
-                            "two": [
-                                StrExpr(
-                                    Pos(0, 0, config_file), Pos(0, 0, config_file), "t1"
-                                ),
-                                StrExpr(
-                                    Pos(0, 0, config_file), Pos(0, 0, config_file), "t2"
-                                ),
-                            ],
+                            "one": SequenceItemsExpr(
+                                [
+                                    PrimitiveExpr(
+                                        Pos(0, 0, config_file),
+                                        Pos(0, 0, config_file),
+                                        "o1",
+                                    ),
+                                    PrimitiveExpr(
+                                        Pos(0, 0, config_file),
+                                        Pos(0, 0, config_file),
+                                        "o2",
+                                    ),
+                                ]
+                            ),
+                            "two": SequenceItemsExpr(
+                                [
+                                    PrimitiveExpr(
+                                        Pos(0, 0, config_file),
+                                        Pos(0, 0, config_file),
+                                        "t1",
+                                    ),
+                                    PrimitiveExpr(
+                                        Pos(0, 0, config_file),
+                                        Pos(0, 0, config_file),
+                                        "t2",
+                                    ),
+                                ]
+                            ),
                         },
                         exclude=[
                             {
-                                "one": StrExpr(
+                                "one": PrimitiveExpr(
                                     Pos(0, 0, config_file), Pos(0, 0, config_file), "o1"
                                 ),
-                                "two": StrExpr(
+                                "two": PrimitiveExpr(
                                     Pos(0, 0, config_file), Pos(0, 0, config_file), "t2"
                                 ),
                             }
                         ],
                         include=[
                             {
-                                "one": StrExpr(
+                                "one": PrimitiveExpr(
                                     Pos(0, 0, config_file), Pos(0, 0, config_file), "o3"
                                 ),
-                                "two": StrExpr(
+                                "two": PrimitiveExpr(
                                     Pos(0, 0, config_file), Pos(0, 0, config_file), "t3"
                                 ),
                             }
@@ -991,7 +1016,7 @@ def test_parse_matrix_with_strategy(assets: pathlib.Path) -> None:
                 Pos(25, 4, config_file),
                 Pos(28, 0, config_file),
                 _specified_fields={"id", "image", "cmd"},
-                inherits=None,
+                mixins=None,
                 id=OptIdExpr(
                     Pos(25, 8, config_file), Pos(25, 14, config_file), "simple"
                 ),
@@ -1154,7 +1179,7 @@ def test_parse_args(assets: pathlib.Path) -> None:
                     config_file,
                 ),
                 _specified_fields={"image", "cmd"},
-                inherits=None,
+                mixins=None,
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 image=OptStrExpr(
@@ -1233,7 +1258,7 @@ def test_parse_enable(assets: pathlib.Path) -> None:
                 _start=Pos(2, 4, config_file),
                 _end=Pos(6, 2, config_file),
                 _specified_fields={"cmd", "id", "preset", "image"},
-                inherits=None,
+                mixins=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "task_a"),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs=None,
@@ -1283,7 +1308,7 @@ def test_parse_enable(assets: pathlib.Path) -> None:
                 _start=Pos(6, 4, config_file),
                 _end=Pos(11, 0, config_file),
                 _specified_fields={"enable", "image", "needs", "cmd", "preset"},
-                inherits=None,
+                mixins=None,
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 title=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 needs={
@@ -1367,7 +1392,7 @@ def test_parse_mixin(assets: pathlib.Path) -> None:
                 Pos(3, 4, config_file),
                 Pos(5, 0, config_file),
                 _specified_fields={"image", "preset"},
-                inherits=None,
+                mixins=None,
                 name=OptStrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
                 image=OptStrExpr(
                     Pos(0, 0, config_file),
@@ -1417,8 +1442,8 @@ def test_parse_mixin(assets: pathlib.Path) -> None:
             ast.Task(
                 _start=Pos(6, 4, config_file),
                 _end=Pos(9, 2, config_file),
-                _specified_fields={"inherits", "cmd"},
-                inherits=[
+                _specified_fields={"mixins", "cmd"},
+                mixins=[
                     StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "basic")
                 ],
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
@@ -1465,8 +1490,8 @@ def test_parse_mixin(assets: pathlib.Path) -> None:
             ast.Task(
                 _start=Pos(9, 4, config_file),
                 _end=Pos(11, 0, config_file),
-                _specified_fields={"inherits", "cmd"},
-                inherits=[
+                _specified_fields={"mixins", "cmd"},
+                mixins=[
                     StrExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), "basic")
                 ],
                 id=OptIdExpr(Pos(0, 0, config_file), Pos(0, 0, config_file), None),
