@@ -3,12 +3,12 @@ import pytest
 from datetime import timedelta
 from neuro_sdk import Client
 from textwrap import dedent
-from typing import Mapping, Optional
+from typing import Mapping, Optional, Tuple, Union
 from typing_extensions import AsyncIterator
 from yarl import URL
 
 from neuro_flow import ast
-from neuro_flow.ast import CacheStrategy
+from neuro_flow.ast import CacheStrategy, InputType
 from neuro_flow.config_loader import BatchLocalCL, ConfigLoader, LiveLocalCL
 from neuro_flow.context import (
     EMPTY_ROOT,
@@ -20,7 +20,13 @@ from neuro_flow.context import (
     sanitize_name,
     setup_inputs_ctx,
 )
-from neuro_flow.expr import EvalError, SimpleOptStrExpr, SimpleStrExpr, StrExpr
+from neuro_flow.expr import (
+    EvalError,
+    PrimitiveExpr,
+    SimpleOptPrimitiveExpr,
+    SimpleOptStrExpr,
+    SimpleStrExpr,
+)
 from neuro_flow.parser import ConfigDir
 from neuro_flow.tokenizer import Pos
 from neuro_flow.types import LocalPath, RemotePath, TaskStatus
@@ -165,6 +171,22 @@ async def test_project_level_defaults_live(
         await cl.close()
 
 
+async def test_project_level_mixins_live(assets: pathlib.Path, client: Client) -> None:
+    ws = assets / "with_project_yaml"
+    config_dir = ConfigDir(
+        workspace=ws,
+        config_dir=ws,
+    )
+    cl = LiveLocalCL(config_dir, client)
+    try:
+        flow = await RunningLiveFlow.create(cl, "live")
+        job = await flow.get_job("test_mixin", {})
+        assert job.image == "mixin-image"
+        assert job.preset == "mixin-preset"
+    finally:
+        await cl.close()
+
+
 async def test_local_remote_path_images(
     client: Client, live_config_loader: ConfigLoader
 ) -> None:
@@ -385,6 +407,19 @@ async def test_pipeline_matrix(batch_config_loader: ConfigLoader) -> None:
         assert task.strategy.fail_fast
 
 
+async def test_matrix_dynamic(batch_config_loader: ConfigLoader) -> None:
+    flow = await RunningBatchFlow.create(
+        batch_config_loader, "batch-matrix-dynamic", "bake-id"
+    )
+
+    assert flow.graph == {
+        "task-1-o1-t1": {},
+        "task-1-o1-t2": {},
+        "task-1-o2-t1": {},
+        "task-1-o2-t2": {},
+    }
+
+
 async def test_pipeline_matrix_with_strategy(batch_config_loader: ConfigLoader) -> None:
     flow = await RunningBatchFlow.create(
         batch_config_loader, "batch-matrix-with-strategy", "bake-id"
@@ -505,10 +540,10 @@ async def test_pipeline_matrix_with_doubles(batch_config_loader: ConfigLoader) -
     )
 
     assert flow.graph == {
-        "task_0_1__0_3": {},
-        "task_0_1__0_5": {},
-        "task_0_2__0_3": {},
-        "task_0_2__0_5": {},
+        "taskN0__0_1__0_3__0": {},
+        "taskN1__0_1__0_5__1": {},
+        "taskN2__0_2__0_3__2": {},
+        "taskN3__0_2__0_5__3": {},
     }
 
 
@@ -580,24 +615,37 @@ async def test_batch_action_default(batch_config_loader: ConfigLoader) -> None:
     assert task.cache == CacheConf(strategy=ast.CacheStrategy.DEFAULT, life_span=1800)
 
 
-def _make_ast_call(args: Mapping[str, str]) -> ast.BaseActionCall:
+def _make_ast_call(
+    args: Mapping[str, Union[bool, int, float, str]]
+) -> ast.BaseActionCall:
     def _make_simple_str_expr(res: Optional[str]) -> SimpleStrExpr:
         return SimpleStrExpr(
             Pos(0, 0, LocalPath("fake")), Pos(0, 0, LocalPath("fake")), res
         )
 
-    def _make_str_expr(res: str) -> StrExpr:
-        return StrExpr(Pos(0, 0, LocalPath("fake")), Pos(0, 0, LocalPath("fake")), res)
+    def _make_primitive_expr(res: Union[bool, int, float, str]) -> PrimitiveExpr:
+        return PrimitiveExpr(
+            Pos(0, 0, LocalPath("fake")), Pos(0, 0, LocalPath("fake")), res
+        )
 
     return ast.BaseActionCall(
         _start=Pos(0, 0, LocalPath("fake")),
         _end=Pos(0, 0, LocalPath("fake")),
         action=_make_simple_str_expr("ws:test"),
-        args={key: _make_str_expr(value) for key, value in args.items()},
+        args={key: _make_primitive_expr(value) for key, value in args.items()},
     )
 
 
-def _make_ast_inputs(args: Mapping[str, Optional[str]]) -> Mapping[str, ast.Input]:
+def _make_ast_inputs(
+    args: Mapping[str, Tuple[Optional[Union[bool, int, float, str]], InputType]]
+) -> Mapping[str, ast.Input]:
+    def _make_opt_primitive_expr(
+        res: Optional[Union[bool, int, float, str]]
+    ) -> SimpleOptPrimitiveExpr:
+        return SimpleOptPrimitiveExpr(
+            Pos(0, 0, LocalPath("fake")), Pos(0, 0, LocalPath("fake")), res
+        )
+
     def _make_opt_str_expr(res: Optional[str]) -> SimpleOptStrExpr:
         return SimpleOptStrExpr(
             Pos(0, 0, LocalPath("fake")), Pos(0, 0, LocalPath("fake")), res
@@ -607,8 +655,9 @@ def _make_ast_inputs(args: Mapping[str, Optional[str]]) -> Mapping[str, ast.Inpu
         key: ast.Input(
             _start=Pos(0, 0, LocalPath("fake")),
             _end=Pos(0, 0, LocalPath("fake")),
-            default=_make_opt_str_expr(value),
+            default=_make_opt_primitive_expr(value[0]),
             descr=_make_opt_str_expr(None),
+            type=value[1],
         )
         for key, value in args.items()
     }
@@ -622,7 +671,7 @@ async def test_setup_inputs_ctx(
         await setup_inputs_ctx(
             EMPTY_ROOT,
             _make_ast_call({"other": "1", "unknown": "2"}),
-            _make_ast_inputs({"expected": None}),
+            _make_ast_inputs({"expected": (None, InputType.STR)}),
         )
 
 
@@ -644,7 +693,9 @@ async def test_batch_action_with_inputs_no_default(
         await setup_inputs_ctx(
             EMPTY_ROOT,
             _make_ast_call({"arg2": "2"}),
-            _make_ast_inputs({"arg1": None, "arg2": "default"}),
+            _make_ast_inputs(
+                {"arg1": (None, InputType.STR), "arg2": ("default", InputType.STR)}
+            ),
         )
 
 
@@ -652,7 +703,9 @@ async def test_batch_action_with_inputs_ok(batch_config_loader: ConfigLoader) ->
     inputs = await setup_inputs_ctx(
         EMPTY_ROOT,
         _make_ast_call({"arg1": "v1", "arg2": "v2"}),
-        _make_ast_inputs({"arg1": None, "arg2": "default"}),
+        _make_ast_inputs(
+            {"arg1": (None, InputType.STR), "arg2": ("default", InputType.STR)}
+        ),
     )
 
     assert inputs == {"arg1": "v1", "arg2": "v2"}
@@ -664,12 +717,32 @@ async def test_batch_action_with_inputs_default_ok(
     inputs = await setup_inputs_ctx(
         EMPTY_ROOT,
         _make_ast_call({"arg1": "v1"}),
-        _make_ast_inputs({"arg1": None, "arg2": "default"}),
+        _make_ast_inputs(
+            {"arg1": (None, InputType.STR), "arg2": ("default", InputType.STR)}
+        ),
     )
 
     assert inputs == {"arg1": "v1", "arg2": "default"}
 
 
+async def test_batch_action_with_inputs_types_do_not_match(
+    batch_config_loader: ConfigLoader,
+) -> None:
+    with pytest.raises(
+        EvalError,
+        match=r"Type of argument 'arg1' do not match to with inputs"
+        r" declared type. Argument has type 'str', declared "
+        r"input type is 'bool'",
+    ):
+        await setup_inputs_ctx(
+            EMPTY_ROOT,
+            _make_ast_call({"arg1": "v1"}),
+            _make_ast_inputs(
+                {"arg1": (None, InputType.BOOL), "arg2": ("default", InputType.STR)}
+            ),
+        )
+
+
 async def test_local_call_with_cache_invalid(
     assets: pathlib.Path,
     client: Client,
@@ -718,7 +791,7 @@ async def test_job_with_live_action(live_config_loader: ConfigLoader) -> None:
     assert job.http_port is None
     assert not job.http_auth
     assert job.entrypoint is None
-    assert job.cmd == "bash -euo pipefail -c 'echo A val 1 B value 2 C'"
+    assert job.cmd == "bash -euo pipefail -c 'echo A val 1 B 2 C'"
     assert job.workdir is None
     assert job.volumes == []
     assert job.tags == {
@@ -1033,6 +1106,26 @@ async def test_batch_with_mixins(batch_config_loader: ConfigLoader) -> None:
     assert task.cmd == "bash -euo pipefail -c 'echo def'"
 
 
+async def test_batch_with_mixins_with_bash_python_properties(
+    batch_config_loader: ConfigLoader,
+) -> None:
+    flow = await RunningBatchFlow.create(
+        batch_config_loader, "batch-mixin-with-bash-python-properties", "bake-id"
+    )
+    task = await flow.get_task((), "task-1", needs={}, state={})
+
+    assert task.image == "ubuntu"
+    assert task.preset == "cpu-micro"
+    assert task.cmd == "bash -euo pipefail -c 'echo abc'"
+
+    task = await flow.get_task(
+        (), "task-2", needs={"task-1": DepCtx(TaskStatus.SUCCEEDED, {})}, state={}
+    )
+    assert task.image == "ubuntu"
+    assert task.preset == "cpu-micro"
+    assert task.cmd == "python3 -uc 'print(\"abc\")'"
+
+
 async def test_batch_with_sub_mixins(batch_config_loader: ConfigLoader) -> None:
     flow = await RunningBatchFlow.create(
         batch_config_loader, "batch-sub-mixin", "bake-id"
@@ -1172,3 +1265,35 @@ async def test_batch_with_project_globals(assets: pathlib.Path, client: Client)
 
     finally:
         await cl.close()
+
+
+async def test_batch_with_project_mixins(assets: pathlib.Path, client: Client) -> None:
+    ws = assets / "with_project_yaml"
+    config_dir = ConfigDir(
+        workspace=ws,
+        config_dir=ws,
+    )
+    cl = BatchLocalCL(config_dir, client)
+    try:
+        flow = await RunningBatchFlow.create(cl, "batch", "bake-id")
+        task = await flow.get_task((), "test_mixin", needs={}, state={})
+        assert task.image == "mixin-image"
+        assert task.preset == "mixin-preset"
+
+        task = await flow.get_task((), "test_mixin_cmd", needs={}, state={})
+        assert task.image == "mixin-image"
+        assert task.cmd == "command -o --option arg1 arg2"
+
+        task = await flow.get_task((), "test_mixin_bash", needs={}, state={})
+        assert task.image == "mixin-image"
+        assert task.cmd == (
+            "bash -euo pipefail -c 'command -o --option arg1 arg2\n"
+            "command2 -o --option arg1 arg2\n'"
+        )
+
+        task = await flow.get_task((), "test_mixin_python", needs={}, state={})
+        assert task.image == "mixin-image"
+        assert task.cmd == "python3 -uc 'print(\"hello neuro-flow\")\n'"
+
+    finally:
+        await cl.close()
